package com.qmth.distributed.print.business.service.impl;

import com.qmth.distributed.print.business.bean.dto.open.PaperConfig;
import com.qmth.distributed.print.business.bean.dto.open.PaperDimension;
import com.qmth.distributed.print.business.bean.dto.open.PaperEvaluation;
import com.qmth.distributed.print.business.bean.dto.open.PaperStructure;
import com.qmth.distributed.print.business.entity.GradeBatchPaper;
import com.qmth.distributed.print.business.entity.GradeModuleDefine;
import com.qmth.distributed.print.business.entity.GradeModuleEvaluation;
import com.qmth.distributed.print.business.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class OpenApiServiceImpl implements OpenApiService {

    @Resource
    private GradeBatchPaperService gradeBatchPaperService;

    @Resource
    private GradePaperService gradePaperService;

    @Resource
    private GradePaperDimensionService gradePaperDimensionService;

    @Resource
    private GradePaperStructService gradePaperStructService;

    @Resource
    private GradeModuleDefineService gradeModuleDefineService;

    @Resource
    private GradeModuleEvaluationService gradeModuleEvaluationService;

    @Override
    public PaperConfig getPaperConfig(String thirdExamId, String gradeCourseCode) {
        GradeBatchPaper gradeBatchPaper = gradeBatchPaperService.findByThirdExamIdAndGradeCourseCode(thirdExamId, gradeCourseCode);
        if (gradeBatchPaper == null) {
            return null;
        }
        return gradePaperService.findBySchoolIdAndPaperNumberAndPaperType(gradeBatchPaper.getSchoolId(), gradeBatchPaper.getPaperNumber(), gradeBatchPaper.getPaperType());
    }

    @Override
    public PaperDimension getPaperDimension(String thirdExamId, String gradeCourseCode) {
        GradeBatchPaper gradeBatchPaper = gradeBatchPaperService.findByThirdExamIdAndGradeCourseCode(thirdExamId, gradeCourseCode);
        if (gradeBatchPaper == null) {
            return null;
        }
        return gradePaperDimensionService.findBySchoolIdAndPaperNumberAndPaperType(gradeBatchPaper.getSchoolId(), gradeBatchPaper.getPaperNumber(), gradeBatchPaper.getPaperType());
    }

    @Override
    public PaperStructure getPaperStructure(String thirdExamId, String gradeCourseCode) {
        GradeBatchPaper gradeBatchPaper = gradeBatchPaperService.findByThirdExamIdAndGradeCourseCode(thirdExamId, gradeCourseCode);
        if (gradeBatchPaper == null) {
            return null;
        }

        return gradePaperStructService.findBySchoolIdAndPaperNumberAndPaperType(gradeBatchPaper.getSchoolId(), gradeBatchPaper.getPaperNumber(), gradeBatchPaper.getPaperType());
    }

    @Override
    public List<PaperEvaluation> getPaperEvaluation(String thirdExamId, String gradeCourseCode) {
        GradeBatchPaper gradeBatchPaper = gradeBatchPaperService.findByThirdExamIdAndGradeCourseCode(thirdExamId, gradeCourseCode);
        if (gradeBatchPaper == null) {
            return null;
        }
        List<GradeModuleDefine> gradeModuleDefineList = gradeModuleDefineService.findBySchoolIdAndPaperNumberAndPaperType(gradeBatchPaper.getSchoolId(), gradeBatchPaper.getPaperNumber(), gradeBatchPaper.getPaperType());
        List<GradeModuleEvaluation> gradeModuleEvaluationList = gradeModuleEvaluationService.findBySchoolIdAndPaperNumberAndPaperType(gradeBatchPaper.getSchoolId(), gradeBatchPaper.getPaperNumber(), gradeBatchPaper.getPaperType());
        // 分组（分隔符_）
        if (!gradeModuleDefineList.isEmpty()) {
            List<PaperEvaluation> paperEvaluations = new ArrayList<>();
            Map<String, List<GradeModuleDefine>> stringListMap = gradeModuleDefineList.stream().collect(Collectors.groupingBy(m -> m.getModuleType() + "_" + m.getInterpret() + "_" + m.getRemark()));
            for (Map.Entry<String, List<GradeModuleDefine>> stringListEntry : stringListMap.entrySet()) {
                String[] key = stringListEntry.getKey().split("_");
                String moduleType = key[0];
                String interpret = key[1];
                String remark = key[3];

                PaperEvaluation paperEvaluation = new PaperEvaluation();
                paperEvaluation.setModuleType(moduleType);
                paperEvaluation.setInterpret(interpret);
                paperEvaluation.setRemark(remark);

                List<PaperEvaluation.SecondaryDimensionLevelDefine> secondaryDimensionLevelDefineArrayList = new ArrayList<>();
                for (GradeModuleDefine gradeModuleDefine : stringListEntry.getValue()) {
                    PaperEvaluation.SecondaryDimensionLevelDefine secondaryDimensionLevelDefine = new PaperEvaluation.SecondaryDimensionLevelDefine();
                    secondaryDimensionLevelDefine.setDefine(gradeModuleDefine.getDefine());
                    secondaryDimensionLevelDefine.setLevel(gradeModuleDefine.getLevel());
                    secondaryDimensionLevelDefine.setMax(gradeModuleDefine.getMax());
                    secondaryDimensionLevelDefine.setMin(gradeModuleDefine.getMin());
                    secondaryDimensionLevelDefine.setScope(gradeModuleDefine.getScope());
                    secondaryDimensionLevelDefineArrayList.add(secondaryDimensionLevelDefine);
                }
                paperEvaluation.setSecondaryDimensionLevelDefineList(secondaryDimensionLevelDefineArrayList);

                if (!gradeModuleEvaluationList.isEmpty()) {
                    List<PaperEvaluation.ModuleEvaluation> moduleEvaluationList = gradeModuleEvaluationList.stream().filter(m -> moduleType.equals(m.getModuleType())).map(m -> {
                        PaperEvaluation.ModuleEvaluation moduleEvaluation = new PaperEvaluation.ModuleEvaluation();
                        moduleEvaluation.setAdvice(m.getAdvice());
                        moduleEvaluation.setLevelCode(m.getLevelCode());
                        moduleEvaluation.setLevelName(m.getLevelName());
                        moduleEvaluation.setResult(m.getResult());
                        moduleEvaluation.setScope(m.getScope());
                        return moduleEvaluation;
                    }).collect(Collectors.toList());
                    paperEvaluation.setModuleEvaluationList(moduleEvaluationList);
                }

                paperEvaluations.add(paperEvaluation);
            }
            return paperEvaluations;
        }
        return null;
    }
}
